'use strict';

// Define constants.
/** @override @constant {Number} The projectile's starting x-position */
Game.prototype.PROJECTILE_X = 62;
/** @override @constant {Number} The projectile's starting y-position */
Game.prototype.PROJECTILE_Y = Game.prototype.SCREEN_HEIGHT - 80;
/** @override @constant {Number} The launcher's x-position */
Game.prototype.LAUNCHER_X = 126;
/** @override @constant {Number} The launcher's y-position */
Game.prototype.LAUNCHER_Y = Game.prototype.SCREEN_HEIGHT - 94;
/** @override @constant {Number} The launch bar's x-position */
Game.prototype.LAUNCH_BAR_X = 194;
/** @override @constant {Number} The x-coordinate at which enemies take the castle and win */
Game.prototype.ENEMY_WIN_X = 332;

/**
 * @override
 * Start the game (as long as all assets are loaded).
 * @returns {Promise} Resolves when all assets are loaded and the game has started
 */
Game.prototype.start = function () {
	return new Promise((function (resolve, reject) {
		this._loadPromise.then((function () {
			this._backgrounds = [
				new Background(this, 0.45, 'back'),
				new Background(this, 1, 'front')
			];
			this._launcher = new ArmLauncher(this, this.LAUNCHER_X, this.LAUNCHER_Y);
			this._projectile = new Projectile(this, this.PROJECTILE_X, this.PROJECTILE_Y);
			this._launchBar = new LaunchBar(this, this.LAUNCH_BAR_X, this.LAUNCH_BAR_Y);
			
			this._prespawnEnemies();
			this._enemySpawnTimer = this._calculateEnemySpawnInterval();
			
			this._lastFrameTime = 0;
			requestAnimationFrame(this._boundUpdate);
			resolve();
		}).bind(this));
	}).bind(this));
};
